package com.dsic.gj_erp.service.txzz;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.sy.DmEmployee;
import com.dsic.gj_erp.bean.tree.ProjectTreeVo;
import com.dsic.gj_erp.bean.txzz.DmProject;
import com.dsic.gj_erp.mapper.txzz.DmProjectMapper;
import com.dsic.gj_erp.service.sy.DmEmployeeService;
import com.dsic.gj_erp.service.sy.SyDictItemService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 船只信息 服务实现类
 * </p>
 *
 * @author xhj
 * @since 2021-03-19
 */
@Service
public class DmProjectServiceImpl extends ServiceImpl<DmProjectMapper, DmProject> implements DmProjectService {

    @Autowired
    DmProjectMapper dmProjectMapper;


    @Autowired
    SyDictItemService dictItemService;

    @Autowired
    RedisTemplate<String, Object> redisTemplate;

    @Autowired
    DmEmployeeService employeeService;

    @Override
    public List<ProjectTreeVo> getProjectTree(Map<String, Object> params) {
        QueryWrapper<DmProject> queryWrapper = new QueryWrapper<DmProject>();
        if (!StringUtils.isBlank((String) params.get("name"))) {
            queryWrapper.like("project", params.get("name"));
        }
        queryWrapper.select("project");
        queryWrapper.apply("IFNULL(project,'')<>''");
        queryWrapper.orderBy(true, true, "finish_flag", "project");
        List<DmProject> list = this.list(queryWrapper);

        //父级
        List<ProjectTreeVo> vo= new ArrayList<>();
        ProjectTreeVo projectTreeVo =new ProjectTreeVo();
        //子级
        List<ProjectTreeVo> projectTreeVoList = new ArrayList<>();

        list.stream().forEach(e->{
            ProjectTreeVo tree=new ProjectTreeVo();
            tree.setId(e.getProject());
            tree.setLabel(e.getProject());
            tree.setParentBm("@@@");
            projectTreeVoList.add(tree);
        });
        projectTreeVo.setId("@@@").setLabel("工程编号").setChildren(projectTreeVoList);
        vo.add(projectTreeVo);
        return  vo;
    }

    /**
     * 条件查询船只
     * @author 张恩铭
     * @param dmProject
     * @return
     */
    @Override
    public ResultBean<List<DmProject>> getByCondition(DmProject dmProject) {
        QueryWrapper<DmProject> queryWrapper = new QueryWrapper<>();

        if(StringUtils.isNotBlank(dmProject.getProject())){
            queryWrapper.likeRight("project",dmProject.getProject());
        }
        if(StringUtils.isNotBlank(dmProject.getProjectName())){
            queryWrapper.likeRight("project_name",dmProject.getProjectName());
        }
        //queryWrapper.apply("IFNULL(project,'')<>''");
        queryWrapper.orderBy(true,true,"finish_flag","project");
        return new ResultBean( this.list(queryWrapper));

    }

    /**
     * 只查所有船号
     * @author 张恩铭
     * @return
     */
    @Override
    public ResultBean<List<DmProject>> getAllProject() {
        ResultBean<List<DmProject>> resultBean = new ResultBean<>();
        QueryWrapper<DmProject> queryWrapper = new QueryWrapper<>();
        //queryWrapper.select("project");
        queryWrapper.select("project","factory_area","classs","owner","project_type","finish_flag");
        //qw.orderByDesc("finish_flag")
        //   qw. orderBy("age");
        queryWrapper.orderBy(true,true,"finish_flag","length(project)","project");
        queryWrapper.apply("IFNULL(project,'')<>''");
        resultBean.setData(this.dmProjectMapper.selectList(queryWrapper));
        return resultBean;
    }

    /**
     * 修改管系部分
     * @param dmProjects
     * @return
     */
    @Override
    public ResultBean<Integer> updateProject(DmProject[] dmProjects) {
//        int i = 0;
//        for (DmProject d : dmProjects) {
//            DmProject temp = new DmProject();
//            temp.setProject(d.getProject());
//            temp.setSproject(d.getSproject());
//            temp.setGjProject(d.getGjProject());
//            temp.setGjOperator(d.getGjOperator());
//            temp.setGjgsStandard(d.getGjgsStandard());
//            temp.setPipeFactor(d.getPipeFactor());
//            temp.setPipeSupport(d.getPipeSupport());
//            temp.setPipeFlat(d.getPipeFlat());
//            temp.setId(d.getId());
//            if(this.dmProjectMapper.updateById(temp) == 1) i++;
//        }
//        ResultBean<Integer> resultBean = new ResultBean<>();
//        resultBean.setData(i);
//        return resultBean;
        ResultBean<Integer> result=this.updatePrivate(dmProjects);
        return result;
    }

    @Transactional
    public ResultBean<Integer> updatePrivate(DmProject[] dmProjects){
        int i = 0;
        for (DmProject d : dmProjects) {
            DmProject temp = new DmProject();
            temp.setProject(d.getProject());
            temp.setSproject(d.getSproject());
            temp.setGjProject(d.getGjProject());
            temp.setGjOperator(d.getGjOperator());
            temp.setGjgsStandard(d.getGjgsStandard());
            temp.setPipeFactor(d.getPipeFactor());
            temp.setPipeSupport(d.getPipeSupport());
            temp.setPipeFlat(d.getPipeFlat());
            temp.setCraftBy(d.getCraftBy());
            temp.setCraftBy2(d.getCraftBy2());
            temp.setCraftBy1(d.getCraftBy1());
            temp.setGjCky(d.getGjCky());
            temp.setGjDey(d.getGjDey());
            temp.setGjFty(d.getGjFty());
            temp.setGjHly(d.getGjHly());
            temp.setGjShy(d.getGjShy());
            temp.setId(d.getId());
            if(this.dmProjectMapper.updateById(temp) == 1) i++;
        }
        ResultBean<Integer> resultBean = new ResultBean<>();
        resultBean.setData(i);
        return resultBean;
    }

    @Override
    public DmProject getProgect(DmProject project) {
        DmProject resultBean ;
        resultBean = this.dmProjectMapper.getProject(project.getProject());
        return resultBean;
    }

    @Override
    public Map mpGetBase(Map<String, Object> params) {
        QueryWrapper<DmProject> qw = new QueryWrapper<DmProject>();
        qw.select("project","factory_area","classs","owner","project_type");
        qw.orderBy(true,true,"finish_flag","length(project)","project");
        qw.apply("IFNULL(project,'')<>''");
        List<DmProject> listTable = this.list(qw);


        //字典
        Map dictMap= new HashMap();
        List<Map<String, Object>> gcqy = dictItemService.getCodeByString("GCQY");
        List<Map<String, Object>> gclx = dictItemService.getCodeByString("GCLX");
        dictMap.put("gcqy",gcqy);
        dictMap.put("gclx",gclx);




        // 配送单位  //配送标识 =1


        Map  map = new HashMap();
        map.put("list",listTable);
        map.put("dict",dictMap);
        return map;
    }

    @Override
    public void saveBaths(HttpServletRequest httpRequest, List<DmProject> dmProjects) {
        //this.saveBatch(dmProjects);

        List<DmEmployee> list=employeeService.list(new QueryWrapper<DmEmployee>()
                .eq("depart_code","01")
                .eq("employee_status","1")
        );
        dmProjects.forEach(p->{
            for(DmEmployee e:list){
                if(StrUtil.isNotEmpty(e.getZone())){  //工艺员
                    if(e.getZone().equals("01")){// '船装';
                        p.setCraftBy(e.getEmployeeName());
                    }
                    if(e.getZone().equals("02")){//'机装'
                        p.setCraftBy1(e.getEmployeeName());
                    }
                    if(e.getZone().equals("03")){//'居装'
                        p.setCraftBy2(e.getEmployeeName());
                    }
                }

                if( null!= e.getTitle()&&e.getTitle().equals("06")){   //sy_dict_item item_code =06= DmEmployee title   计调员
                    p.setTxOperator(e.getEmployeeName());
                }
            }
            p.setBathNo2("1001");//船装批号
            p.setBathNo3("T1001");//T船装批号
            p.setBathNo4("2001");//机装批号
            p.setBathNo5("T2001");//T机装批号
            p.setBathNo6("3001");//居装批号
            p.setBathNo7("T3001");//T居装批号
        });
        this.saveAll(dmProjects);
    }

    @Transactional
    public void saveAll(List<DmProject> dmProjects){
        this.saveBatch(dmProjects);
    }
}
